 
# Virtual Lab - optional components - dumb terminal
# $Id$
#
# Copyright (C) 2009, Jack Whitham
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#


import sys

from twisted.internet import reactor
from twisted.internet.protocol import Protocol
from terminput import setTransport


class DumbTerminal(Protocol):
    """A Protocol for communication with 
    a virtual lab UART.

    This Protocol implements an interactive session. When the
    local user presses a key, it is sent to the board via the
    virtual lab UART. Any data that is returned is (by default)
    written directly to stdout.

    The typical usage for this class is, given a VlabClientFactory vlf:
    >>> vl = yield vlf.getChannel()
    >>> yield vl.connect("boardname")
    >>> 
    >>> from vlab import terminal
    >>> uart = terminal.DumbTerminal()
    >>> terminal.setTransport(uart)
    >>> yield vl.openUART(0, uart)

    setTransport(uart) arranges for key presses to be
    sent to the DumbTerminal. Unlike the usual StandardIO classes,
    this input is unbuffered and received in raw mode on both Windows
    and Unix platforms.

    You should override the methods of DumbTerminal if you want a
    slightly smarter terminal, e.g. to implement an escape character
    (see write) or filter out unprintable characters (dataReceived)."""

    def __init__(self):
        self.transport = None

    def dataReceived(self, data):
        """This method is called by the transport whenever data
        is received over the network. 
        
        The default behaviour is
        >>> sys.stdout.write(data)
        >>> sys.stdout.flush()"""

        sys.stdout.write(data)
        sys.stdout.flush()

    def write(self, data):
        """This method is called whenever data is received from the
        local user (i.e. key press). 
        
        The default behaviour is
        >>> if ( self.transport != None ):
        >>>     self.transport.write(data)
        """

        if ( self.transport != None ):
            self.transport.write(data)

    def makeConnection(self, transport):
        self.transport = transport

